//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.ResourceTimeline
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// Create some test resources
			Appointment app = null;
			Random r = new Random();

			_calendar.Resources.Add(new Resource { Name = "Resource #1" });
			_calendar.Resources.Add(new Resource { Name = "Resource #2" });
			_calendar.Resources.Add(new Resource { Name = "Resource #3" });

			for (int i = 0; i < 20; i++)
			{
				app = new Appointment();
				app.StartTime = _calendar.Date + TimeSpan.FromDays(r.Next(10) + 3);
				app.EndTime = app.StartTime + TimeSpan.FromHours(24 + r.Next(24));
				app.HeaderText = "Resource " + i.ToString();
				app.Resources.Add(_calendar.Resources[r.Next(3)]);
				_calendar.Schedule.Items.Add(app);
			}

			InitTimeline();
			UpdateTimeline();
			_calendar.HScroll += (s, e) =>
			{
				if (e.Type == ScrollEventType.EndScroll)
					UpdateTimeline();
			};

			_calendar.SetViewport(new DateTime((long)timeline.SelectionStart),
				new DateTime((long)timeline.SelectionEnd), true);

			timeline.ValueChanged += (s, e) =>
			{
				_calendar.SetViewport(new DateTime((long)timeline.SelectionStart),
					new DateTime((long)timeline.SelectionEnd), true);
				timeline.Refresh();
			};

			timeline.DrawBackground += (s, e) =>
			{
				var bounds = e.Bounds;

				// Draw days
				var date = _calendar.Date;
				while (date < _calendar.EndDate)
				{
					double prc = (double)(date.Ticks - _calendar.Date.Ticks) / (_calendar.EndDate.Ticks - _calendar.Date.Ticks);

					int x = bounds.X + (int)(prc * bounds.Width);
					using (var pen = new Pen(_calendar.ResourceViewSettings.Style.BorderLeftColor))
						e.Graphics.DrawLine(pen, x, bounds.Top, x, bounds.Bottom);
					using (var brush = new SolidBrush(_calendar.ResourceViewSettings.Style.HeaderTextColor))
						e.Graphics.DrawString(date.ToString("MMM"), _calendar.ResourceViewSettings.Style.HeaderFont, brush, x, bounds.Top);

					date = date.AddDays(DateTime.DaysInMonth(date.Year, date.Month) - date.Day + 1);
				}
			};
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			UpdateTimeline();
		}

		/// <summary>
		/// Initialize the timeline from the Calendar control.
		/// </summary>
		void InitTimeline()
		{
			DateTime start = _calendar.Date;
			DateTime end = _calendar.EndDate;

			timeline.Minimum = start.Ticks;
			timeline.Maximum = end.Ticks;

			timeline.MinimumSelectionLength = (timeline.Maximum - timeline.Minimum) / 30;
			timeline.Step = TimeSpan.TicksPerDay;
			timeline.SelectionStart = timeline.Minimum;
			timeline.SelectionEnd = timeline.SelectionStart + timeline.MinimumSelectionLength;
		}

		/// <summary>
		/// Updates the timeline from the Calendar control.
		/// </summary>
		void UpdateTimeline()
		{
			timeline.SelectionStart = _calendar.GetFirstVisibleDate().Ticks;
			timeline.SelectionEnd = _calendar.GetLastVisibleDate().Ticks;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}


		private string _appPath;
	}
}
